@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Time::C;
-$Time::C::VERSION = '0.022';
+$Time::C::VERSION = '0.023';
 # ABSTRACT: Convenient time manipulation.
 
 use overload (
@@ -459,7 +459,16 @@ method _tm ($t: $new_tm = undef) :lvalue {
 }
 
 
-method string ($t: $new_str = undef, :$format = undef, :$locale = 'C', :$strict = 1) :lvalue {
+# need to parse the @args specially since F::P can't handle the signature
+method string ($t: @args) :lvalue {
+    my ($new_str, $format, $locale, $strict) = (undef, undef, 'C', 1);
+    if (@args % 2) { $new_str = shift @args; }
+    my %args = @args;
+    $format = delete $args{format} if exists $args{format};
+    $locale = delete $args{locale} if exists $args{locale};
+    $strict = delete $args{strict} if exists $args{strict};
+    croak sprintf "In method string: no such named parameter: %s", sort keys %args if %args;
+
     $t->{tz} = 'UTC' if not defined $t->{tz};
 
     my $setter = sub {
@@ -506,7 +515,7 @@ method string ($t: $new_str = undef, :$format = undef, :$locale = 'C', :$strict
 }
 
 
-method strftime ($t: $new_str = undef, :$format = undef, :$locale = 'C', :$strict = 1) :lvalue { $t->string(@_); }
+method strftime ($t: @args) :lvalue { $t->string(@args); }
 
 
 method year ($t: $new_year = undef) :lvalue {
@@ -749,7 +758,7 @@ Time::C - Convenient time manipulation.
 
 =head1 VERSION
 
-version 0.022
+version 0.023
 
 =head1 SYNOPSIS
 
