@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Time::C::Util;
-$Time::C::Util::VERSION = '0.022';
+$Time::C::Util::VERSION = '0.023';
 # ABSTRACT: Utility functions for Time::C and friends.
 
 use Carp qw/ croak /;
@@ -57,6 +57,16 @@ fun get_locale($type, $locale) {
         $ret = $loc_db->{t_fmt}->{$locale};
     } elsif ($type eq 'time_ampm') {
         $ret = $loc_db->{r_fmt}->{$locale};
+    } elsif ($type eq 'era') {
+        $ret = $loc_db->{era}->{$locale} // [];
+    } elsif ($type eq 'era_datetime') {
+        $ret = $loc_db->{era_d_t_fmt}->{$locale} // $loc_db->{d_t_fmt}->{$locale};
+    } elsif ($type eq 'era_time') {
+        $ret = $loc_db->{era_t_fmt}->{$locale} // $loc_db->{t_fmt}->{$locale};
+    } elsif ($type eq 'era_date') {
+        $ret = $loc_db->{era_d_fmt}->{$locale} // $loc_db->{d_fmt}->{$locale};
+    } elsif ($type eq 'digits') {
+        $ret = $loc_db->{alt_digits}->{$locale} // [];
     } else { croak "Unknown locale type: $type."; }
 
     croak "Value for locale type $type in locale $locale is undefined."
@@ -79,7 +89,7 @@ Time::C::Util - Utility functions for Time::C and friends.
 
 =head1 VERSION
 
-version 0.022
+version 0.023
 
 =head1 AUTHOR
 
