@@ -1,13 +1,14 @@
 use strict;
 use warnings;
 package Time::P;
-$Time::P::VERSION = '0.022';
+$Time::P::VERSION = '0.023';
 # ABSTRACT: Parse times from strings.
 
 use Carp qw/ croak /;
 use Exporter qw/ import /;
 use Function::Parameters;
 use Data::Munge qw/ list2re /;
+use List::Util qw/ uniq /;
 
 use Time::C::Util qw/ get_fmt_tok get_locale /;
 
@@ -45,6 +46,24 @@ my %parser; %parser = (
     },
     '%d'  => fun () { qr"(?<d>[0-9][0-9])"; },
     '%-d' => fun () { qr"(?<d>[0-9][0-9]?)"; },
+    '%EC' => fun (:$locale) {
+        my @eras = _get_eras(period => $locale);
+        return $parser{'%C'}->() if not @eras;
+        my $re = list2re(@eras);
+        return qr"(?<EC>$re)";
+    },
+    '%Ec' => fun (:$locale) { _compile_fmt(get_locale(era_datetime => $locale), locale => $locale); },
+    '%EX' => fun (:$locale) { _compile_fmt(get_locale(era_time => $locale), locale => $locale); },
+    '%Ex' => fun (:$locale) { _compile_fmt(get_locale(era_date => $locale), locale => $locale); },
+    '%EY' => fun (:$locale) {
+        my @eras = _get_eras(full => $locale);
+        return $parser{'%Y'}->() if not @eras;
+
+        my @ret = map { my $re = join "", _compile_fmt($_, locale => $locale); qr/$re/ } uniq @eras;
+        my $full_re = join "|", @ret;
+        return qr/$full_re/;
+    },
+    '%Ey' => fun () { qr"(?<Ey>[0-9]+)"; },
     '%e'  => fun () { qr"(?:\s(?<e>[0-9])|(?<e>[0-9][0-9]))"; },
     '%-e' => fun () { qr"(?<e>[0-9][0-9]?)"; },
     '%F'  => fun () {
@@ -70,6 +89,106 @@ my %parser; %parser = (
     '%m'  => fun () { qr"(?<m>[0-9][0-9])"; },
     '%-m' => fun () { qr"(?<m>[0-9][0-9]?)"; },
     '%n'  => fun () { qr"\s+"; },
+    '%OC' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%d'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OC." if @d < 100;
+        my $re = list2re(@d);
+        return qr"(?<OC>$re)";
+    },
+    '%Od' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%d'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %Od." if @d < 32;
+        my $re = list2re(@d);
+        return qr"(?<Od>$re)";
+    },
+    '%Oe' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%e'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %Oe." if @d < 32;
+        my $re = list2re(@d);
+        return qr"(?<Oe>$re)";
+    },
+    '%OH' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%H'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OH." if @d < 24;
+        my $re = list2re(@d);
+        return qr"(?<OH>$re)";
+    },
+    '%OI' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%I'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OI." if @d < 13;
+        my $re = list2re(@d);
+        return qr"(?<OI>$re)";
+    },
+    '%OM' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%M'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OM." if @d < 60;
+        my $re = list2re(@d);
+        return qr"(?<OM>$re)";
+    },
+    '%Om' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%m'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %Om." if @d < 13;
+        my $re = list2re(@d);
+        return qr"(?<Om>$re)";
+    },
+    '%Op' => fun (:$locale) { $parser{'%p'}->(locale => $locale); }, # one %c spec in my_MM locale erroneously says %Op instead of %p
+    '%OS' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%S'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OS." if @d < 60;
+        my $re = list2re(@d);
+        return qr"(?<OS>$re)";
+    },
+    '%OU' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%U'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OU." if @d < 54;
+        my $re = list2re(@d);
+        return qr"(?<OU>$re)";
+    },
+    '%Ou' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%u'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %Ou." if @d < 8;
+        my $re = list2re(@d);
+        return qr"(?<Ou>$re)";
+    },
+    '%OV' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%V'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OV." if @d < 54;
+        my $re = list2re(@d);
+        return qr"(?<OV>$re)";
+    },
+    '%OW' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%W'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %OW." if @d < 54;
+        my $re = list2re(@d);
+        return qr"(?<OW>$re)";
+    },
+    '%Ow' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%w'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %Ow." if @d < 7;
+        my $re = list2re(@d);
+        return qr"(?<Ow>$re)";
+    },
+    '%Oy' => fun (:$locale) {
+        my @d = @{ get_locale(digits => $locale) };
+        return $parser{'%y'}->() if not @d;
+        croak "Not enough digits in alt_digits for $locale to represent %Oy." if @d < 100;
+        my $re = list2re(@d);
+        return qr"(?<Oy>$re)";
+    },
+    '%P'  => fun (:$locale) { $parser{'%p'}->(locale => $locale); }, # a few %r specs in some locales erroneously say %P instead of %p (wal_ET, ur_PK, pa_PK, iw_IL, he_IL, en_GB, dv_MV, cy_GB)
     '%p'  => fun (:$locale) {
         my @am_pm = @{ get_locale(am_pm => $locale) };
         return () unless @am_pm;
@@ -100,8 +219,8 @@ my %parser; %parser = (
     '%W'  => fun () { qr"(?<W>[0-9][0-9])"; },
     '%-W' => fun () { qr"(?<W>[0-9][0-9]?)"; },
     '%w'  => fun () { qr"(?<w>[0-9])"; },
-    '%Y'  => fun () { qr"(?<Y>[0-9]{4})"; },
-    '%-Y' => fun () { qr"(?<Y>[0-9]{1,4})"; },
+    '%Y'  => fun () { qr"(?<Y>-?[0-9]{4})"; },
+    '%-Y' => fun () { qr"(?<Y>-?[0-9]{1,4})"; },
     '%y'  => fun () { qr"(?<y>[0-9][0-9])"; },
     '%-y' => fun () { qr"(?<y>[0-9][0-9]?)"; },
     '%Z'  => fun () { qr"(?<Z>\S+)"; },
@@ -114,15 +233,19 @@ fun strptime ($str, $fmt, :$locale = 'C', :$strict = 1, :$struct = {}) {
     my %parse = ();
 
     my @res = _compile_fmt($fmt, locale => $locale);
+
+    croak "Could not match '%s' using '%s'.", $str, $fmt if not @res;
+
     @res = (qr/^/, @res, qr/$/) if $strict;
 
-    while (@res and $str =~ m/\G$res[0]/gc) {
+    my $re;
+    while (defined ($re = shift @res) and $str =~ m/\G$re/gc) {
+        warn "matched with $re\n" if DEBUG;
         %parse = (%parse, %+);
-        shift @res;
     }
 
     if (@res) {
-        croak sprintf "Could not match '%s' using '%s'. Match failed at position %d.", $str, $fmt, pos($str);
+        croak sprintf "Could not match '%s' using '%s'. Match failed at position %d (%s) while trying to match with /%s/.", $str, $fmt, pos($str), substr($str, pos($str)), $re;
     }
 
     $struct = { %$struct, _coerce_struct(\%parse, $struct, locale => $locale) };
@@ -138,26 +261,38 @@ fun _compile_fmt ($fmt, :$locale) {
     # _get_tok will increment $pos for us
     while (defined(my $tok = get_fmt_tok($fmt, $pos))) {
         if (exists $parser{$tok}) {
-            push @res, $parser{$tok}->(locale => $locale);
+            my @p_res = $parser{$tok}->(locale => $locale);
+            warn "pushing @p_res to list\n" if DEBUG;
+            push @res, @p_res;
         } elsif ($tok =~ /^%/) {
             croak "Unsupported format specifier: $tok";
         } else {
-            push @res, qr/\Q$tok\E/;
+            my $re = qr/\Q$tok\E/;
+            warn "pushing $re to list\n" if DEBUG;
+            push @res, $re;
         }
     }
 
+    warn "returning @res\n" if DEBUG;
     return @res;
 }
 
-fun _coerce_struct ($struct, $orig, :$locale) {
-    # First, if we know the epoch, great
-    my $epoch = $struct->{'s'};
+fun _get_mday ($struct, :$locale) {
+    my $mday = $struct->{'d'};
+    if (not defined $mday) { $mday = $struct->{'e'}; }
+    if (not defined $mday and defined(my $Od = $struct->{Od})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $mday = _get_index($Od, @d);
+    }
+    if (not defined $mday and defined(my $Oe = $struct->{Oe})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $mday = _get_index($Oe, @d);
+    }
 
-    # Then set up as many date bits we know about
-    #  year + day of year
-    #  year + month + day of month
-    #  year + week + day of week
+    return $mday;
+}
 
+fun _get_year ($struct, :$locale) {
     my $wyear = 0;
     my $year = $struct->{'Y'};
     if (not defined $year) {
@@ -182,25 +317,58 @@ fun _coerce_struct ($struct, $orig, :$locale) {
             $wyear = 1;
         }
     }
+    if (not defined $year) {
+        my $Ey = $struct->{Ey};
+        my $EC = $struct->{EC};
 
-    my $yday = $struct->{'j'};
+        if (defined $EC) {
+            my @eras = @{ get_locale(era => $locale) };
+            foreach my $era (@eras) {
+                my @fields = split /:/, $era;
+                next if $EC ne $fields[4];
 
-    my $month = $struct->{'m'};
-    if (not defined $month) {
-        if (defined $struct->{'B'}) {
-            $month = _get_index($struct->{'B'}, @{ get_locale(months => $locale) }) + 1;
-        } elsif (defined $struct->{'b'}) {
-            $month = _get_index($struct->{'b'}, @{ get_locale(months_abbr => $locale) }) + 1;
-        } 
+                my %s = strptime($fields[2], "%-Y/%m/%d");
+                $s{year}++ if $s{year} < 1;
+                $year = $s{year};
+                $year += $Ey - $fields[1] if defined $Ey;
+                $year-- if not defined $Ey;
+                last;
+            }
+        } elsif (defined $Ey) {
+            my @eras = @{ get_locale(era => $locale) };
+            foreach my $era (@eras) {
+                my @fields = split /:/, $era;
+
+                my %s = strptime($fields[2], "%-Y/%m/%d");
+                $s{year}++ if $s{year} < 1;
+                require Time::C;
+                next if $s{year} > Time::C->now_utc()->year;
+
+                $year = $s{year} + $Ey - $fields[1];
+                last;
+            }
+        }
+    }
+    if (not defined $year and defined(my $Oy = $struct->{Oy})) {
+        my @d = @{ get_locale(digits => $locale) };
+        if (defined(my $C = $struct->{C})) {
+            $year = $C * 100;
+        } elsif (defined(my $OC = $struct->{OC})) {
+            $year = _get_index($OC, @d) * 100;
+        } else {
+            $year = 1900;
+        }
+        $year += _get_index($Oy, @d);
+        if (not defined $struct->{C}) {
+            require Time::C;
+            if ($year < (Time::C->now_utc()->year - 50)) { $year += 100; }
+        }
     }
 
-    my $mday = $struct->{'d'};
-    if (not defined $mday) { $mday = $struct->{'e'}; }
-
-    my $u_week = $struct->{'U'};
-    my $w_week = $struct->{'W'};
-    my $v_week = $struct->{'V'};
+    return ($year, $wyear);
+}
 
+fun _get_wday($struct, :$locale) {
     my $wday = $struct->{'u'} // $struct->{'w'};
 
     if (not defined $wday) {
@@ -210,8 +378,151 @@ fun _coerce_struct ($struct, $orig, :$locale) {
             $wday = _get_index($struct->{'a'}, @{ get_locale(weekdays_abbr => $locale) });
         }
     }
+    if (not defined $wday) {
+        if (defined(my $Ou = $struct->{Ou})) {
+            my @d = @{ get_locale(digits => $locale) };
+            $wday = _get_index($Ou, @d);
+        } elsif (defined(my $Ow = $struct->{Ow})) {
+            my @d = @{ get_locale(digits => $locale) };
+            $wday = _get_index($Ow, @d);
+        }
+    }
     $wday = 7 if defined $wday and $wday == 0;
 
+    return $wday;
+}
+
+fun _get_u_week ($struct, :$locale) {
+    my $u_week = $struct->{U};
+
+    if (not defined $u_week and defined(my $OU = $struct->{OU})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $u_week = _get_index($OU, @d);
+    }
+
+    return $u_week;
+}
+
+fun _get_w_week ($struct, :$locale) {
+    my $w_week = $struct->{W};
+
+    if (not defined $w_week and defined(my $OW = $struct->{OW})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $w_week = _get_index($OW, @d);
+    }
+
+    return $w_week;
+}
+
+fun _get_v_week ($struct, :$locale) {
+    my $v_week = $struct->{V};
+
+    if (not defined $v_week and defined(my $OV = $struct->{OV})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $v_week = _get_index($OV, @d);
+    }
+
+    return $v_week;
+}
+
+fun _get_month ($struct, :$locale) {
+    my $month = $struct->{'m'};
+    if (not defined $month) {
+        if (defined $struct->{'B'}) {
+            $month = _get_index($struct->{'B'}, @{ get_locale(months => $locale) }) + 1;
+        } elsif (defined $struct->{'b'}) {
+            $month = _get_index($struct->{'b'}, @{ get_locale(months_abbr => $locale) }) + 1;
+        }
+    }
+    if (not defined $month and defined(my $Om = $struct->{Om})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $month = _get_index($Om, @d);
+    }
+
+    return $month;
+}
+
+fun _get_hour ($struct, :$locale) {
+    my $hour = $struct->{'H'};
+    if (not defined $hour) { $hour = $struct->{'k'}; }
+    if (not defined $hour) {
+        $hour = $struct->{'I'} // $struct->{'l'};
+        if (defined $hour and length $struct->{'p'}) {
+            if (_get_index($struct->{'p'}, @{ get_locale(am_pm => $locale) })) {
+                # PM
+                if ($hour < 12) { $hour += 12; }
+            } else {
+                # AM
+                if ($hour == 12) { $hour = 0; }
+            }
+        }
+    }
+
+    if (not defined $hour and defined(my $OH = $struct->{OH})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $hour = _get_index($OH, @d);
+    } elsif (not defined $hour and defined(my $OI = $struct->{OI})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $hour = _get_index($OI, @d);
+        if (length $struct->{p}) {
+            if (_get_index($struct->{p}, @{ get_locale(am_pm => $locale) })) {
+                # PM
+                if ($hour < 12) { $hour += 12; }
+            } else {
+                # AM
+                if ($hour == 12) { $hour = 0; }
+            }
+        }
+    }
+
+    return $hour;
+}
+
+fun _get_minute ($struct, :$locale) {
+    my $min = $struct->{'M'};
+
+    if (not defined $min and defined(my $OM = $struct->{OM})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $min = _get_index($OM, @d);
+    }
+
+    return $min;
+}
+
+fun _get_second ($struct, :$locale) {
+    my $sec = $struct->{'S'};
+
+    if (not defined $sec and defined(my $OS = $struct->{OS})) {
+        my @d = @{ get_locale(digits => $locale) };
+        $sec = _get_index($OS, @d);
+    }
+
+    return $sec;
+}
+
+fun _coerce_struct ($struct, $orig, :$locale) {
+    # First, if we know the epoch, great
+    my $epoch = $struct->{'s'};
+
+    # Then set up as many date bits we know about
+    #  year + day of year
+    #  year + month + day of month
+    #  year + week + day of week
+
+    my ($year, $wyear) = _get_year($struct, locale => $locale);
+
+    my $yday = $struct->{'j'};
+
+    my $month = _get_month($struct, locale => $locale);
+
+    my $mday = _get_mday($struct, locale => $locale);
+
+    my $u_week = _get_u_week($struct, locale => $locale);
+    my $w_week = _get_w_week($struct, locale => $locale);
+    my $v_week = _get_v_week($struct, locale => $locale);
+
+    my $wday = _get_wday($struct, locale => $locale);
+
     if (not defined $w_week and defined $u_week) {
         $w_week = $u_week;
         if (not defined $wday) { $wday = 7; } # if no wday defined, should set to first day of week, and since the u_week starts at sunday, wday = 7
@@ -244,24 +555,11 @@ fun _coerce_struct ($struct, $orig, :$locale) {
 
     # Next try to set up time bits -- these are pretty easy in comparison
 
-    my $hour = $struct->{'H'};
-    if (not defined $hour) { $hour = $struct->{'k'}; }
-    if (not defined $hour) {
-        $hour = $struct->{'I'} // $struct->{'l'};
-        if (defined $hour and length $struct->{'p'}) {
-            if (_get_index($struct->{'p'}, @{ get_locale(am_pm => $locale) })) {
-                # PM
-                if ($hour < 12) { $hour += 12; }
-            } else {
-                # AM
-                if ($hour == 12) { $hour = 0; }
-            }
-        }
-    }
+    my $hour = _get_hour($struct, locale => $locale);
 
-    my $min = $struct->{'M'};
+    my $min = _get_minute($struct, locale => $locale);
 
-    my $sec = $struct->{'S'};
+    my $sec = _get_second($struct, locale => $locale);
 
     # And last see if we have some timezone or at least offset info
 
@@ -307,6 +605,13 @@ fun _get_index ($needle, @haystack) {
     croak "Could not find $needle in the list.";
 }
 
+fun _get_eras ($type, $locale) {
+    my @eras = @{ get_locale(era => $locale) };
+    my @ret = map { my @fields = split /:/; $type eq 'period' ? $fields[4] : $fields[5] } @eras;
+
+    return @ret;
+}
+
 1;
 
 __END__
@@ -321,7 +626,7 @@ Time::P - Parse times from strings.
 
 =head1 VERSION
 
-version 0.022
+version 0.023
 
 =head1 SYNOPSIS
 
@@ -418,6 +723,40 @@ Equivalent to C<%m/%d/%y>, e.g. C<10/30/16> - this is used mostly in the US. Int
 
 If you specify C<%-d>, it will be 1 or 2 digits if the day of the month is low enough.
 
+=item C<%E*>
+
+The C<%E*> format specifiers that are supported are:
+
+=over
+
+=item C<%Ec>
+
+Similar to C<%c>, but using an alternate way of displaying the year. See C<%EY>, C<%EC>, and C<%Ey>.
+
+=item C<%EC>
+
+Similarly to C<%C>, it (usually) describes a period longer than 1 year, depending on the locale. There are only a few locales that define this: C<ja_JP>, C<lo_LA>, and C<th_TH>; in other locales it functions like C<%C>.
+
+Taking the C<ja_JP> locale as an example, it defines a number of periods, ostensibly relating to when they got a new emperor, and these periods are what the C<%EC> represents rather than centuries like C<%C>.
+
+=item C<%EX>
+
+Similar to C<%x>, but using an alternate way of displaying the time. There are only a few locales that define this: C<lo_LA>, and C<th_TH>; in other locales it functions like C<%X>.
+
+=item C<%Ex>
+
+Similar to C<%X>, but using an alternate way of displaying the year. There are only a few locales that define this: C<ja_JP>, C<lo_LA>, and C<th_TH>; in other locales it functions like C<%x>. See C<%EY>, C<%EC>, and C<%Ey> for details on how the year is displayed.
+
+=item C<%EY>
+
+Similarly to C<%Y>, it describes a year fully, but depends on the C<%EC> and possibly the C<%Ey> of the locale. There are only a few locales that define this: C<ja_JP>, C<lo_LA>, and C<th_TH>; in other locales it functions like C<%Y>.
+
+=item C<%Ey>
+
+Similarly to C<%y>, it describes the number of years since the period, only C<%y>'s period is C<%C>, while C<%Ey>'s period is C<%EC>. There are only a few locales that define this: C<ja_JP>, C<lo_LA>, and C<th_TH>; in other locales it functions like C<%y>.
+
+=back
+
 =item C<%e>
 
 1/2 digit day of month, space padded, e.g. C<30>.
@@ -490,6 +829,44 @@ If you specify C<%-m>, it will be 1 or 2 digits if the month is low enough.
 
 Arbitrary whitespace, like C<m/\s+/> - if used as a formatting specifier rather than a parsing specifier, it will result in a C<\n> (i.e. a newline).
 
+=item C<%O*>
+
+The C<%O*> format specifiers work like their non-C<O> counterparts, except they use an alternate set of digits for representing the number depending on the locale. Not all C<%O*> specifiers are actually supported by all locales, as some only define numbers up to a certain point. Some don't specify an alternate set of digits at all, in which case they should work I<exactly> like their non-C<O> counterparts.
+
+The C<%O*> format specifiers that are supported are:
+
+=over
+
+=item C<%OC>
+
+=item C<%Od>
+
+=item C<%Oe>
+
+=item C<%OH>
+
+=item C<%OI>
+
+=item C<%OM>
+
+=item C<%Om>
+
+=item C<%OS>
+
+=item C<%OU>
+
+=item C<%Ou>
+
+=item C<%OV>
+
+=item C<%OW>
+
+=item C<%Ow>
+
+=item C<%Oy>
+
+=back
+
 =item C<%p>
 
 Matches the locale version of C<a.m.> or C<p.m.>, if the locale has that. Otherwise matches the empty string.
